
package org.mule.modules.rest.config;

import java.util.List;
import org.apache.commons.lang.StringUtils;
import org.mule.config.spring.MuleHierarchicalBeanDefinitionParserDelegate;
import org.mule.config.spring.factories.MessageProcessorChainFactoryBean;
import org.mule.config.spring.util.SpringXMLUtils;
import org.mule.modules.rest.processors.RouterMessageProcessor;
import org.mule.util.TemplateParser;
import org.springframework.beans.MutablePropertyValues;
import org.springframework.beans.PropertyValue;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.support.ManagedList;
import org.springframework.beans.factory.xml.BeanDefinitionParser;
import org.springframework.beans.factory.xml.ParserContext;
import org.springframework.util.xml.DomUtils;
import org.w3c.dom.Element;

public class RouterDefinitionParser
    implements BeanDefinitionParser
{

    /**
     * Mule Pattern Info
     * 
     */
    private TemplateParser.PatternInfo patternInfo;

    public RouterDefinitionParser() {
        patternInfo = TemplateParser.createMuleStyleParser().getStyle();
    }

    public BeanDefinition parse(Element element, ParserContext parserContent) {
        BeanDefinitionBuilder builder = BeanDefinitionBuilder.rootBeanDefinition(RouterMessageProcessor.class.getName());
        String configRef = element.getAttribute("config-ref");
        if ((configRef!= null)&&(!StringUtils.isBlank(configRef))) {
            builder.addPropertyValue("moduleObject", configRef);
        }
        if ((element.getAttribute("templateUri")!= null)&&(!StringUtils.isBlank(element.getAttribute("templateUri")))) {
            builder.addPropertyValue("templateUri", element.getAttribute("templateUri"));
        }
        Element getElement = DomUtils.getChildElementByTagName(element, "get");
        if (getElement!= null) {
            String text = getElement.getAttribute("text");
            if ((text!= null)&&(!StringUtils.isBlank(text))) {
                builder.addPropertyValue("get", text);
            } else {
                BeanDefinitionBuilder getBeanDefinitionBuilder = BeanDefinitionBuilder.rootBeanDefinition(MessageProcessorChainFactoryBean.class);
                BeanDefinition getBeanDefinition = getBeanDefinitionBuilder.getBeanDefinition();
                parserContent.getRegistry().registerBeanDefinition(generateChildBeanName(getElement), getBeanDefinition);
                getElement.setAttribute("name", generateChildBeanName(getElement));
                getBeanDefinitionBuilder.setSource(parserContent.extractSource(getElement));
                getBeanDefinitionBuilder.setScope(BeanDefinition.SCOPE_SINGLETON);
                List getList = parserContent.getDelegate().parseListElement(getElement, getBeanDefinitionBuilder.getBeanDefinition());
                parserContent.getRegistry().removeBeanDefinition(generateChildBeanName(getElement));
                builder.addPropertyValue("get", getBeanDefinition);
            }
        }
        Element putElement = DomUtils.getChildElementByTagName(element, "put");
        if (putElement!= null) {
            String text = putElement.getAttribute("text");
            if ((text!= null)&&(!StringUtils.isBlank(text))) {
                builder.addPropertyValue("put", text);
            } else {
                BeanDefinitionBuilder putBeanDefinitionBuilder = BeanDefinitionBuilder.rootBeanDefinition(MessageProcessorChainFactoryBean.class);
                BeanDefinition putBeanDefinition = putBeanDefinitionBuilder.getBeanDefinition();
                parserContent.getRegistry().registerBeanDefinition(generateChildBeanName(putElement), putBeanDefinition);
                putElement.setAttribute("name", generateChildBeanName(putElement));
                putBeanDefinitionBuilder.setSource(parserContent.extractSource(putElement));
                putBeanDefinitionBuilder.setScope(BeanDefinition.SCOPE_SINGLETON);
                List putList = parserContent.getDelegate().parseListElement(putElement, putBeanDefinitionBuilder.getBeanDefinition());
                parserContent.getRegistry().removeBeanDefinition(generateChildBeanName(putElement));
                builder.addPropertyValue("put", putBeanDefinition);
            }
        }
        Element postElement = DomUtils.getChildElementByTagName(element, "post");
        if (postElement!= null) {
            String text = postElement.getAttribute("text");
            if ((text!= null)&&(!StringUtils.isBlank(text))) {
                builder.addPropertyValue("post", text);
            } else {
                BeanDefinitionBuilder postBeanDefinitionBuilder = BeanDefinitionBuilder.rootBeanDefinition(MessageProcessorChainFactoryBean.class);
                BeanDefinition postBeanDefinition = postBeanDefinitionBuilder.getBeanDefinition();
                parserContent.getRegistry().registerBeanDefinition(generateChildBeanName(postElement), postBeanDefinition);
                postElement.setAttribute("name", generateChildBeanName(postElement));
                postBeanDefinitionBuilder.setSource(parserContent.extractSource(postElement));
                postBeanDefinitionBuilder.setScope(BeanDefinition.SCOPE_SINGLETON);
                List postList = parserContent.getDelegate().parseListElement(postElement, postBeanDefinitionBuilder.getBeanDefinition());
                parserContent.getRegistry().removeBeanDefinition(generateChildBeanName(postElement));
                builder.addPropertyValue("post", postBeanDefinition);
            }
        }
        Element deleteElement = DomUtils.getChildElementByTagName(element, "delete");
        if (deleteElement!= null) {
            String text = deleteElement.getAttribute("text");
            if ((text!= null)&&(!StringUtils.isBlank(text))) {
                builder.addPropertyValue("delete", text);
            } else {
                BeanDefinitionBuilder deleteBeanDefinitionBuilder = BeanDefinitionBuilder.rootBeanDefinition(MessageProcessorChainFactoryBean.class);
                BeanDefinition deleteBeanDefinition = deleteBeanDefinitionBuilder.getBeanDefinition();
                parserContent.getRegistry().registerBeanDefinition(generateChildBeanName(deleteElement), deleteBeanDefinition);
                deleteElement.setAttribute("name", generateChildBeanName(deleteElement));
                deleteBeanDefinitionBuilder.setSource(parserContent.extractSource(deleteElement));
                deleteBeanDefinitionBuilder.setScope(BeanDefinition.SCOPE_SINGLETON);
                List deleteList = parserContent.getDelegate().parseListElement(deleteElement, deleteBeanDefinitionBuilder.getBeanDefinition());
                parserContent.getRegistry().removeBeanDefinition(generateChildBeanName(deleteElement));
                builder.addPropertyValue("delete", deleteBeanDefinition);
            }
        }
        Element patchElement = DomUtils.getChildElementByTagName(element, "patch");
        if (patchElement!= null) {
            String text = patchElement.getAttribute("text");
            if ((text!= null)&&(!StringUtils.isBlank(text))) {
                builder.addPropertyValue("patch", text);
            } else {
                BeanDefinitionBuilder patchBeanDefinitionBuilder = BeanDefinitionBuilder.rootBeanDefinition(MessageProcessorChainFactoryBean.class);
                BeanDefinition patchBeanDefinition = patchBeanDefinitionBuilder.getBeanDefinition();
                parserContent.getRegistry().registerBeanDefinition(generateChildBeanName(patchElement), patchBeanDefinition);
                patchElement.setAttribute("name", generateChildBeanName(patchElement));
                patchBeanDefinitionBuilder.setSource(parserContent.extractSource(patchElement));
                patchBeanDefinitionBuilder.setScope(BeanDefinition.SCOPE_SINGLETON);
                List patchList = parserContent.getDelegate().parseListElement(patchElement, patchBeanDefinitionBuilder.getBeanDefinition());
                parserContent.getRegistry().removeBeanDefinition(generateChildBeanName(patchElement));
                builder.addPropertyValue("patch", patchBeanDefinition);
            }
        }
        if ((element.getAttribute("method")!= null)&&(!StringUtils.isBlank(element.getAttribute("method")))) {
            builder.addPropertyValue("method", element.getAttribute("method"));
        }
        if ((element.getAttribute("uri")!= null)&&(!StringUtils.isBlank(element.getAttribute("uri")))) {
            builder.addPropertyValue("uri", element.getAttribute("uri"));
        }
        BeanDefinition definition = builder.getBeanDefinition();
        definition.setAttribute(MuleHierarchicalBeanDefinitionParserDelegate.MULE_NO_RECURSE, Boolean.TRUE);
        MutablePropertyValues propertyValues = parserContent.getContainingBeanDefinition().getPropertyValues();
        if (parserContent.getContainingBeanDefinition().getBeanClassName().equals("org.mule.config.spring.factories.PollingMessageSourceFactoryBean")) {
            propertyValues.addPropertyValue("messageProcessor", definition);
        } else {
            if (parserContent.getContainingBeanDefinition().getBeanClassName().equals("org.mule.enricher.MessageEnricher")) {
                propertyValues.addPropertyValue("enrichmentMessageProcessor", definition);
            } else {
                PropertyValue messageProcessors = propertyValues.getPropertyValue("messageProcessors");
                if ((messageProcessors == null)||(messageProcessors.getValue() == null)) {
                    propertyValues.addPropertyValue("messageProcessors", new ManagedList());
                }
                List listMessageProcessors = ((List) propertyValues.getPropertyValue("messageProcessors").getValue());
                listMessageProcessors.add(definition);
            }
        }
        return definition;
    }

    protected String getAttributeValue(Element element, String attributeName) {
        if (!StringUtils.isEmpty(element.getAttribute(attributeName))) {
            return element.getAttribute(attributeName);
        }
        return null;
    }

    private String generateChildBeanName(Element element) {
        String id = SpringXMLUtils.getNameOrId(element);
        if (StringUtils.isBlank(id)) {
            String parentId = SpringXMLUtils.getNameOrId(((Element) element.getParentNode()));
            return ((("."+ parentId)+":")+ element.getLocalName());
        } else {
            return id;
        }
    }

}
